package com.crankuptheamps.examples;

import java.lang.Thread;

import com.crankuptheamps.client.*;
import com.crankuptheamps.client.exception.*;

/** EX01AMPSConsoleSubscriber
*
* This is a minimalist way of subscribing to a topic in AMPS. The
* program flow is simple:
*
* * Connect to AMPS
* * Logon
* * Subscribe to all messages published on the "messages" topic 
* * Output the messages to the console
*
* This sample doesn't include error handling or connection
* retry logic.
*
* (c) 2014-2016 60East Technologies, Inc.  All rights reserved.
* This file is a part of the AMPS Evaluation Kit.
*/

public class EX01AMPSConsoleSubscriber {

  // The location of the AMPS server.
  private static final String uri_ = "tcp://127.0.0.1:9007/amps/json";

  /**
   * main method.
   * 
   * @param args -- No command line options read.
   */
   public static void main(String[] args)
   {
        System.out.println("Starting the console subscriber.");

        Client client = new Client("ConsoleSubscriber");

        try
        {
              // connect to the AMPS server and logon
              client.connect(uri_);
              client.logon();


              // subscribe to the messages topic with a timeout of 5000.
              // when a message arrives, print the message.
              
              MessageStream ms = client.subscribe("messages");
              try
              {
                for (Message m : ms)
                {
                   System.out.println(m.getData());
                }
              }
              finally  // close the message stream to release the subscription
              {  ms.close(); }
             
       }
       catch (AMPSException e)
       {
          System.err.println(e.getLocalizedMessage());
          e.printStackTrace(System.err);

       }
  }
}
